#include "StdAfx.h"
#include "Registry.h"
#include "resource.h"
#include "builddesktopimage.h"
#include "MainHelperQ.h"
#include "GdiplusWrapper.h"
#include "AOTTools.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern int GetWPPositionByImageSize(CString cstrImagePath, UINT iHorzRes, UINT iVertRes);

Gdiplus::Bitmap* CutImagePartAndAdjust(Gdiplus::Bitmap* image, CRect ecran, int x, int xMax, int y, int yMax, CRect dest)
{
    Bitmap* image2 = new Bitmap(ecran.Width() /xMax, ecran.Height() / yMax, PixelFormat32bppARGB);
    Graphics* graph2 = Graphics::FromImage(image2);
	CPoint point;
	point.x = ecran.Width() * x / xMax;
	point.y = ecran.Height() * y / yMax;
	graph2->DrawImage(image, 0, 0, point.x, point.y, ecran.Width() / xMax, ecran.Height() / yMax, Unit::UnitPixel);
	delete graph2;
    Bitmap* image3 = new Bitmap(dest.Width(), dest.Height(), PixelFormat32bppARGB);
    Graphics* graph3 = Graphics::FromImage(image3);
	graph3->DrawImage(image2, 0, 0, dest.Width(), dest.Height());

	delete graph3;
	delete image2;
	return image3;
}

CBuildDesktopImage::CBuildDesktopImage()
{
    // Screen Resolution //
    m_nVirtualScreenWidth = GetSystemMetrics(SM_CXVIRTUALSCREEN);
    m_nVirtualScreenHeight = GetSystemMetrics(SM_CYVIRTUALSCREEN);
    HDC hDC = ::GetDC(NULL);

    RefreshMonitors();

    m_hDC = ::CreateCompatibleDC(hDC);
    m_hBitmap = CreateCompatibleBitmap(hDC, m_nVirtualScreenWidth, m_nVirtualScreenHeight);
    ::SelectObject(m_hDC, m_hBitmap);

    COLORREF crDeskColor = GetSysColor(COLOR_DESKTOP);
    CBrush fillBrush;
    fillBrush.CreateSolidBrush(crDeskColor);
    CRect rect(0, 0, m_nVirtualScreenWidth, m_nVirtualScreenHeight);
    ::FillRect(m_hDC, &rect, fillBrush.operator HBRUSH());
}

CBuildDesktopImage::~CBuildDesktopImage(void)
{
    if (m_hBitmap)
    {
        DeleteDC(m_hDC);
        DeleteObject(m_hBitmap);
        m_hBitmap = NULL;
        m_hDC = NULL;
    }

    DeleteMonitors();
}

BOOL CBuildDesktopImage::SaveBitmap(CString szPath)
{
    HANDLE hFile;
    DWORD dwNumWritten;

    hFile = CreateFile(szPath, GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
    if (hFile == INVALID_HANDLE_VALUE)
        return FALSE;

    BITMAPINFO bmi;
    ZeroMemory(&bmi, sizeof(BITMAPINFO));
    bmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
    bmi.bmiHeader.biBitCount = 0;

    if (!::GetDIBits(m_hDC, m_hBitmap, 0, 0, NULL, &bmi, DIB_RGB_COLORS))
        return FALSE;

    BITMAPFILEHEADER bmHdr;

    bmHdr.bfType = 0x4d42;   // 'BM' WINDOWS_BITMAP_SIGNATURE
    bmHdr.bfSize = (((3 * bmi.bmiHeader.biWidth + 3) & ~3) * bmi.bmiHeader.biHeight) + sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER);
    bmHdr.bfReserved1 = bmHdr.bfReserved2 = 0;
    bmHdr.bfOffBits = (DWORD) sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER);

    bmi.bmiHeader.biCompression = BI_RGB;
    // Writing Bitmap File Header ////
    WriteFile(hFile, &bmHdr, sizeof(BITMAPFILEHEADER), &dwNumWritten, NULL);

    WriteFile(hFile, &bmi.bmiHeader, sizeof(BITMAPINFOHEADER), &dwNumWritten, NULL);

    BYTE* pData = new BYTE[bmi.bmiHeader.biSizeImage + 5];
    if (!::GetDIBits(m_hDC, m_hBitmap, 0, bmi.bmiHeader.biHeight, pData, &bmi, DIB_RGB_COLORS))
        return FALSE;
    if (pData != NULL)
        WriteFile(hFile, pData, bmi.bmiHeader.biSizeImage, &dwNumWritten, NULL);

    CloseHandle(hFile);
    delete (pData);

    return TRUE;
}

#define HIMETRIC_INCH 2540

BOOL CBuildDesktopImage::WriteImage(Gdiplus::Bitmap* image, CRect ecran)
{
    if (m_hBitmap)
    {
        // Image Height and Width

        SetStretchBltMode(m_hDC, HALFTONE);

        Graphics* hBitmapDC = new Graphics(m_hDC);

		if (ecran.left >= 0)
        {
            if (ecran.top >= 0)
            {
                hBitmapDC->DrawImage(image, ecran.left, ecran.top, ecran.Width(), ecran.Height());
            }
            else
            {
                hBitmapDC->DrawImage(image, ecran.left, ecran.top, ecran.Width(), ecran.Height());
                hBitmapDC->DrawImage(image, ecran.left, m_nVirtualScreenHeight + ecran.top, ecran.Width(), ecran.Height());
            }
        }
        else
        {
            if (ecran.top >= 0)
            {
                hBitmapDC->DrawImage(image, ecran.left, ecran.top, ecran.Width(), ecran.Height());
                hBitmapDC->DrawImage(image, m_nVirtualScreenWidth + ecran.left, ecran.top, ecran.Width(), ecran.Height());
            }
            else
            {
                hBitmapDC->DrawImage(image, ecran.left, ecran.top, ecran.Width(), ecran.Height());
                hBitmapDC->DrawImage(image, ecran.left, m_nVirtualScreenHeight + ecran.top, ecran.Width(), ecran.Height());
                hBitmapDC->DrawImage(image, m_nVirtualScreenWidth + ecran.left, ecran.top, ecran.Width(), ecran.Height());
                hBitmapDC->DrawImage(image, m_nVirtualScreenWidth + ecran.left, m_nVirtualScreenHeight + ecran.top, ecran.Width(), ecran.Height());
            }
        }
/*		Color col;
		col.SetFromCOLORREF(RGB(128,0,128));
	    SolidBrush* pBrush = new SolidBrush(col);
		ecran.DeflateRect(40, 40);
		hBitmapDC->FillEllipse(pBrush, ecran.left, ecran.top, ecran.Width(), ecran.Height());
		delete pBrush;
*/        delete hBitmapDC;
    }
    else 
        return FALSE;
	return TRUE;
}

Gdiplus::Bitmap* CBuildDesktopImage::SetImage(CString img, int style, CRect ecran)
{
    LONG lWidth = 0,lHeight = 0;

    Image* hMainBitmap = Image::FromFile(img);
    if (hMainBitmap != NULL)
    {
        lWidth = hMainBitmap->GetWidth();
        lHeight = hMainBitmap->GetHeight();
        if (lWidth > 0 && lHeight > 0)
        {
            CRect rectangle1, rectangle2;

            Bitmap* image2 = new Bitmap(ecran.Width(), ecran.Height(), PixelFormat32bppARGB);
            Graphics* graph2 = Graphics::FromImage(image2);
            SolidBrush* solidBrush = new SolidBrush(GetSysColor(COLOR_DESKTOP));
            CRect Rect(0, 0, ecran.Width(), ecran.Height());
            graph2->FillRectangle(solidBrush, 0, 0, ecran.Width(), ecran.Height());
            delete solidBrush;

            if (style == 5)
            {
                style = GetWPPositionByImageSize(img, ecran.Width(), ecran.Height());
            }
            if (style == 0)
            {
                // Tiled
                rectangle1.top = 0;
                rectangle1.left = 0;
                rectangle1.bottom = lHeight;
                rectangle1.right = lWidth;
                for (int xPos = 0; xPos < ecran.Width(); xPos += lWidth)
                {
                    for (int yPos = 0; yPos < ecran.Height(); yPos += lHeight)
                    {
                        graph2->DrawImage(hMainBitmap, xPos, yPos, rectangle1.Width(), rectangle1.Height());
                    }
                }
            }
            if (style == 1)
            {
                // Center

                int imgWidth = lWidth;
                int imgHeight = lHeight;
                if (ecran.Height() < lHeight || ecran.Width() < lWidth)
                {
                    imgWidth = (lWidth * ecran.Height() < lHeight * ecran.Width()) ? lWidth * ecran.Height() / lHeight : lWidth;
                    imgHeight = (lWidth * ecran.Height() < lHeight * ecran.Width()) ? lHeight * imgWidth / lWidth : lHeight;
                }
                if (imgWidth > ecran.Width())
                {
                    imgWidth = ecran.Width();
                    imgHeight = lHeight * ecran.Width() / lWidth;
                }
                rectangle1.top = 0;
                rectangle1.left = 0;
                rectangle1.bottom = imgHeight;
                rectangle1.right = imgWidth;

                graph2->DrawImage(hMainBitmap, (ecran.Width() - rectangle1.Width()) / 2, (ecran.Height() - rectangle1.Height()) / 2, rectangle1.Width(), rectangle1.Height());
            }

            if (style == 2)
            {
                // Stretch
                graph2->DrawImage(hMainBitmap, 0, 0, ecran.Width(), ecran.Height());
            } //Stretch

            if (style == 3)
            {
                // Adjust
                int imgWidth = ecran.Width();
                int imgHeight = lHeight* ecran.Width() / lWidth;
                if (imgHeight > ecran.Height())
                {
                    imgHeight = ecran.Height();
                    imgWidth = lWidth * ecran.Height() / lHeight;
                }
                rectangle1.top = 0;
                rectangle1.left = 0;
                rectangle1.bottom = imgHeight;
                rectangle1.right = imgWidth;
                if (ecran.Width() > rectangle1.Width())
                {
                    graph2->DrawImage(hMainBitmap, (ecran.Width() - rectangle1.Width()) / 2, (ecran.Height() - rectangle1.Height()) / 2, rectangle1.Width(), rectangle1.Height());
                }
                else
                {
                    graph2->DrawImage(hMainBitmap, (ecran.Width() - rectangle1.Width()) / 2, (ecran.Height() - rectangle1.Height()) / 2, rectangle1.Width(), rectangle1.Height());
                }
            } //Adjust

            if (style == 4)
            {
                // Adjust from outside
                int imgWidth = ecran.Width();
                int imgHeight = lHeight* ecran.Width() / lWidth;
                if (imgHeight < ecran.Height())
                {
                    imgHeight = ecran.Height();
                    imgWidth = lWidth * ecran.Height() / lHeight;
                }
                rectangle1.top = 0;
                rectangle1.left = 0;
                rectangle1.bottom = imgHeight;
                rectangle1.right = imgWidth;
                if (ecran.Width() > rectangle1.Width())
                {
                    graph2->DrawImage(hMainBitmap, (ecran.Width() - rectangle1.Width()) / 2, (ecran.Height() - rectangle1.Height()) / 2, rectangle1.Width(), rectangle1.Height());
                }
                else
                {
                    graph2->DrawImage(hMainBitmap, (ecran.Width() - rectangle1.Width()) / 2, (ecran.Height() - rectangle1.Height()) / 2, rectangle1.Width(), rectangle1.Height());
                }
            } //Adjust from outside
            delete graph2;
            delete hMainBitmap;
			return image2;
        }
    }
	return NULL;
}
